# Generated by Django 4.0 on 2022-09-24 23:16

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserExternalData',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('external_id', models.IntegerField(verbose_name='External ID')),
                ('fullname', models.CharField(max_length=200, verbose_name='Full name')),
                ('user_rel', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='user_external_data_rel', to='auth.user', verbose_name='User')),
            ],
            options={
                'verbose_name': "User's external data",
                'verbose_name_plural': 'Users external data',
            },
        ),
    ]



from django.contrib.auth import get_user_model
from django.db.models import ObjectDoesNotExist

from rest_framework import permissions
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.response import Response

from drf_spectacular.utils import extend_schema

from shared_security.api.serializers.users import UserInformationSerializer
from shared_security.api.serializers.errors import ErrorSerializer


class UserViewSet(viewsets.ViewSet):
    """
    User registration and edition endpoints
    """

    @extend_schema(
        methods=["GET"],
        responses={
            200: UserInformationSerializer,
            403: ErrorSerializer,
            422: ErrorSerializer
        },
        summary="Returns the information of a given user from token"
                         )
    @action(detail=True, methods=['GET'], permission_classes=[permissions.IsAuthenticated])
    def retrieve_user(self, request, **kwargs):
        """
        This endpoint returns the information of a given user from token obtained from auth
        microservice
        """

        user = request.user

        user_info_serializer = UserInformationSerializer({
            "username": user.username,
        })

        return Response(user_info_serializer.data, status=200)


from django.contrib import admin
from shared_security import models
from django.contrib.auth import get_user_model
from django.contrib.auth.admin import UserAdmin as AuthUserAdmin


class UserExternalDataInline(admin.StackedInline):
    model = models.UserExternalData
    max_num = 1
    can_delete = False

class UserAdmin(AuthUserAdmin):
    inlines = [UserExternalDataInline,]
    
    
# unregister old user admin
admin.site.unregister(get_user_model())
# register new user admin
admin.site.register(get_user_model(), UserAdmin)



from django.db import models
from django.conf import settings
from django.utils.translation import gettext_lazy as _


class UserExternalData(models.Model):
    """
    Store external data from auth microservice user model
    """

    user_rel = models.OneToOneField(settings.AUTH_USER_MODEL, related_name='user_external_data_rel',
                                    on_delete=models.CASCADE, verbose_name=_("User"), null=False, blank=False)
    external_id = models.IntegerField(null=False, verbose_name=_("External ID"))
    fullname = models.CharField(max_length=200, verbose_name=_("Full name"))

    class Meta:
        verbose_name = _("User's external data")
        verbose_name_plural = _('Users external data')

    def __str__(self):
        return _("External data ") + self.user_rel.username



from rest_framework.authentication import TokenAuthentication
from rest_framework import exceptions

from django.contrib.auth.backends import ModelBackend
from django.contrib.auth import get_user_model

from shared_security.helpers import (
    get_user_jwt_from_microservice, load_django_setting)
from shared_security.models import UserExternalData

import jwt


def check_user_admin_role(user_data):
    if user_data != None and "rol" in user_data and "id" in user_data["rol"] and user_data["rol"]["id"] == 12:
        return True
    elif user_data == None:
        return True

    return False


def create_new_user(user_model, username, user_data=None):
    user = user_model.objects.create(username=username)
    # TODO: at the moment harcoded to the user rol with id 12
    if check_user_admin_role(user_data):
        user.is_staff = True
        user.is_superuser = True
        user.save()
    
    if user_data:
        UserExternalData.objects.create(
            user_rel=user, external_id=user_data["id"], fullname=user_data["fullname"])
    return user


def update_user(user):
    user.is_superuser = True
    user.is_staff = True
    user.save()


class ExternalTokenAuthentication(TokenAuthentication):
    keyword = 'Bearer'  # token type

    def get_data_from_token(self, key):
        JWT_SECRET_KEY = load_django_setting('JWT_SECRET_KEY')
        decoded_token = jwt.decode(
            key, JWT_SECRET_KEY, algorithms=["HS256"])
        return {
            "user_id": decoded_token["id"]
        }

    def authenticate_credentials(self, key):
        # decode JWT.
        try:
            token_data = self.get_data_from_token(key)
        except jwt.ExpiredSignatureError:
            raise exceptions.AuthenticationFailed('Token expired')
        except jwt.InvalidTokenError:
            raise exceptions.AuthenticationFailed('Invalid token')
        user_model = get_user_model()
        SECURITY_USE_PERSISTENT_USERS = load_django_setting(
            'SECURITY_USE_PERSISTENT_USERS', True)
        # TODO: at the moment harcoded to the root user
        # add request to get user data from id or token
        if SECURITY_USE_PERSISTENT_USERS:
            try:
                # TODO: at the moment all user logged at least one time is considered as a superuser
                user = user_model.objects.get(username="root")
                update_user(user)
            except user_model.DoesNotExist:
                user = create_new_user(user_model, "root")
        else:
            user = user_model(
                username="root", is_superuser=True, is_staff=True)
        return user, key


class ExternalAuthenticationBackend(ModelBackend):

    def authenticate(self, request, username=None, password=None, **kwargs):

        # Get the user information from external jwt auth microservice if he can be authenticated
        response = get_user_jwt_from_microservice(username, password)
        if response is None:
            return None

        login_data = response['user']
        user_model = get_user_model()
        SECURITY_USE_PERSISTENT_USERS = load_django_setting(
            'SECURITY_USE_PERSISTENT_USERS', True)
        
        if SECURITY_USE_PERSISTENT_USERS:
            try:
                # TODO: at the moment all user logged at least one time is considered as a superuser
                user = user_model.objects.get(username=login_data["username"])
                update_user(user)
            except user_model.DoesNotExist:
                user = create_new_user(
                    user_model, login_data["username"], login_data)
        else:
            # user = user_model(pk=1,
            #     username=login_data["username"], is_superuser=True, is_staff=True)
            return None
        return user

    def get_user(self, user_id):
        user_model = get_user_model()
        
        SECURITY_USE_PERSISTENT_USERS = load_django_setting(
            'SECURITY_USE_PERSISTENT_USERS', True)
        
        if SECURITY_USE_PERSISTENT_USERS:
            try:
                return user_model.objects.get(pk=user_id)
            except user_model.DoesNotExist:
                return None
        else:
            return None



from rest_framework import serializers

class ErrorSerializer(serializers.Serializer):
    details = serializers.CharField(help_text="Error details")

class SuccessSerializer(serializers.Serializer):
    details = serializers.CharField(help_text="Success details")


from rest_framework.authentication import get_authorization_header

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import gettext_lazy as _

import requests


def load_django_setting(setting_name, default_value=None):
    setting_value = getattr(settings, setting_name, default_value)

    if setting_value == None:
        raise ImproperlyConfigured(_(f"Setting {setting_name} is not configured."))
        
    return setting_value


def get_data_from_microservice(request, path):
    try:
        headers = {"Authorization": get_authorization_header(request)}
        r = requests.get(path, headers=headers)
        r.raise_for_status()
    except requests.exceptions.HTTPError as err:
        return err.response.json()

    except Exception as err:
        raise Exception(err)

    return r.json()


def get_user_jwt_from_microservice(username, password):
    try:
        headers = {"Content-Type": "application/json"}
        path = settings.USER_MICROSERVICE_LOGIN_URI
        r = requests.post(
            path, json={"email": username, "password": password}, headers=headers)
        r.raise_for_status()
    except requests.exceptions.HTTPError as err:
        # return err.response.json()
        return None

    except Exception as err:
        # raise Exception(err)
        return None

    return r.json()
